Package: IHW
Title: Independent Hypothesis Weighting
Version: 1.40.0
Authors@R: c(person("Nikos", "Ignatiadis", email = "nikos.ignatiadis01@gmail.com", role = c("aut", "cre")),
             person("Wolfgang", "Huber", role = "aut"))
Description: Independent hypothesis weighting (IHW) is a multiple
        testing procedure that increases power compared to the method
        of Benjamini and Hochberg by assigning data-driven weights to
        each hypothesis. The input to IHW is a two-column table of
        p-values and covariates. The covariate can be any
        continuous-valued or categorical variable that is thought to be
        informative on the statistical properties of each hypothesis
        test, while it is independent of the p-value under the null
        hypothesis.
Depends: R (>= 3.3.0)
License: Artistic-2.0
Imports: methods, slam, lpsymphony, fdrtool, BiocGenerics
Suggests: ggplot2, dplyr, gridExtra, scales, DESeq2, airway, testthat,
        Matrix, BiocStyle, knitr, rmarkdown, devtools
VignetteBuilder: knitr
biocViews: ImmunoOncology, MultipleComparison, RNASeq
RoxygenNote: 7.1.1
Config/pak/sysreqs: coinor-symphony coinor-libsymphony-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:42:33 UTC
RemoteUrl: https://github.com/bioc/IHW
RemoteRef: RELEASE_3_23
RemoteSha: ade9922140017b59ea7a8ed1b6867880fdb1443a
NeedsCompilation: no
Packaged: 2026-04-29 15:04:51 UTC; root
Author: Nikos Ignatiadis [aut, cre],
  Wolfgang Huber [aut]
Maintainer: Nikos Ignatiadis <nikos.ignatiadis01@gmail.com>
Built: R 4.6.0; ; 2026-04-29 15:22:27 UTC; unix
