# Generated by roxygen2: do not edit by hand

S3method(ihw,DESeqResults)
S3method(ihw,default)
S3method(ihw,formula)
export(get_bh_threshold)
export(groups_by_filter)
export(ihw)
exportMethods(adj_pvalues)
exportMethods(alpha)
exportMethods(as.data.frame)
exportMethods(covariate_type)
exportMethods(covariates)
exportMethods(groups_factor)
exportMethods(m_groups)
exportMethods(nbins)
exportMethods(nfolds)
exportMethods(nrow)
exportMethods(plot)
exportMethods(pvalues)
exportMethods(regularization_term)
exportMethods(rejected_hypotheses)
exportMethods(rejections)
exportMethods(show)
exportMethods(thresholds)
exportMethods(weighted_pvalues)
exportMethods(weights)
import(methods)
importFrom(BiocGenerics,as.data.frame)
importFrom(fdrtool,gcmlcm)
importFrom(lpsymphony,lpsymphony_solve_LP)
importFrom(methods,show)
importFrom(slam,simple_triplet_matrix)
importFrom(slam,simple_triplet_zero_matrix)
